function [Ht,numfactors] = ogarch(Data,p,q,r,m,varthresh)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%INPUT:   Data       - n*k matrix of returns,
%         p          - order of the GARCH terms for individual asset estimation
%         q          - order of the ARCH terms for individual asset estimation
%         r          - autoregressive model order of an ARMA(R,M) mean equation with GARCH dynamics for error volatility 
%         m          - moving-average model order of an ARMA(R,M) mean equation with GARCH dynamics for error volatility 
%         varthresh  - threshold percentage of the desired level of explained
%                      variation explained by the first several principal components in
%                      the Orthogonal GARCH model
%OUTPUT:  Ht         - estimated time-varying covariance matrix of returns
%         numfactors - number of principal components (factors) corresponding to the given threshold
%                      percentage of explained variation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[~,k] = size(Data);

[W,SCORES,EIG,~] = princomp(Data); %getting the matrices of principal components (factors) and factor loadings 
%and the cumulative percentage of explained variation 
numfactors       = find(cumsum(EIG)./sum(EIG) >= varthresh,1,'first'); %determining the number of principal components corresponding 
%the explained variation percentage threshold
WEIGHTS          = W(1:numfactors,:);
PRCOMP           = SCORES(:,1:numfactors);

%modelling the time-varying volatility of principal components
htMat            = zeros(1,numfactors);
Specs            = garchset('Distribution', 'Gaussian', 'Display', 'off', 'VarianceModel', 'GARCH', 'P', p, 'Q', q,...
                            'R', r, 'M', m);
for i=1:numfactors
    [Parameters, ~, ~, ~, ht, ~] = garchfit(Specs,PRCOMP(:,i));
    [HForec,~]                   = garchpred(Parameters,PRCOMP(:,i),1);
    if isnan(HForec)
        htMat(:,i) = ht(end);
    else
        htMat(:,i) = HForec;
    end
end

PRCV = diag(htMat);

%determining errors
if numfactors<k
    errors = Data - PRCOMP*WEIGHTS;
    Omega  = diag(mean(errors.^2));
else
    Omega  = zeros(k);
end

%calculating the time-dependent covariance matrix
Ht = WEIGHTS'*PRCV*WEIGHTS + Omega;
end